/*
 * @(#)TetrisP.h
 *
 * Copyright 1993 - 2008  David A. Bagley, bagleyd@tux.org
 *
 * Taken from GENERIC X-BASED TETRIS
 *
 * Copyright 1992 - 1995  Q. Alex Zhao, azhao@cc.gatech.edu
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Private header file for Tetris */

#ifndef _TetrisP_h
#define _TetrisP_h

#include "xwin.h"
#include "timer.h"
#include "polyomino.h"
#include "Tetris.h"

#ifdef WINVER
extern void destroyGame(HBRUSH brush);
extern void resizeGame(TetrisWidget w);
extern void sizeGame(TetrisWidget w);
extern void initializeGame(TetrisWidget w, HBRUSH brush);
extern void exposeGame(TetrisWidget w);
extern void hideGame(TetrisWidget w);
extern void clearGame(TetrisWidget w);
extern void enterGame(TetrisWidget w);
extern void leaveGame(TetrisWidget w);
extern void moveGameInput(TetrisWidget w, int direction);
extern void soundGame(TetrisWidget w);
extern Boolean incrementGameLevel(TetrisWidget w);
extern Boolean decrementGameLevel(TetrisWidget w);
extern Boolean incrementGameUnits(TetrisWidget w);
extern Boolean decrementGameUnits(TetrisWidget w);
extern Boolean incrementGameColumns(TetrisWidget w);
extern Boolean decrementGameColumns(TetrisWidget w);
extern void fixedfillGame(TetrisWidget w);
extern void suddenAppearGame(TetrisWidget w);
extern Boolean diagonalGame(TetrisWidget w);
extern Boolean mixedGame(TetrisWidget w);
extern void bonusGame(TetrisWidget w);
extern void cwGame(TetrisWidget w);
extern void plainPieceGame(TetrisWidget w);
extern void falldownGame(TetrisWidget w);
extern void pauseGame(TetrisWidget w);
extern void showNextGame(TetrisWidget w);
extern void fillGame(TetrisWidget w, int n);
extern void setGame(TetrisWidget w, int reason);
extern void setGameNext(TetrisWidget w, int reason,
	int randomNumber, int randomRotation, int randomReflection,
	int units, Boolean diagonal, Boolean mixed, Boolean newBonus);

#else

/* This gets around C's inability to do inheritance */
typedef struct _TetrisClassPart {
	int         ignore;
} TetrisClassPart;

typedef struct _TetrisClassRec {
	CoreClassPart coreClass;
	TetrisClassPart tetrisClass;
} TetrisClassRec;

extern TetrisClassRec tetrisClassRec;
#endif

#ifndef BUMPSOUND
#define BUMPSOUND "bump"
#endif

#define SYMBOL ':'

#define NUM_FLASHES 4

#define MAX_SIDES 4
#define MIN_LEVELS 1
#define MAX_LEVELS 20
#define BG_SHADES 5

#define THRESHOLD(x)	((x)*10)

/*** types ***/

typedef enum {FALL, DROP, MOVE_LEFT, MOVE_RIGHT, ROTATE, REFLECT} move_t;

typedef struct {
	int         units;
	Boolean     diagonal, mixed, bonus;
} Spec;

typedef struct {
	int         units, polyNumber;
	int         xpos, ypos;
	int         size, colorNumber;
	int         bonus, randomRotation, randomReflection;
	long        randomNumber;
} Thing;

typedef struct {
	int         pmid, cid;
} Field;

typedef struct _TetrisPart {
	Thing       curThing, nextThing;
	Field      *field;
	GC          thingGC[MAX_START_POLYOMINOES];
	Boolean     fixedFill, suddenAppear, cw, plainPiece, showNext;
	Boolean     sound, resetTime, firstFall, paused, gotNew;
	int         level, fill, completedRows;
	int         startLevel, score, newRows;
	Spec        current, next;
	int         units, columns, rows;
	Boolean     diagonal, mixed, bonus;
	int         delay, numberPolyominoes, pauseStart;
	int         currentPosition, dropPaused;
	Boolean     fall, started, quit, act;
	Boolean     mono, reverse, focus;
	Boolean     scoreOnly, versionOnly;
	int         boxSize;
	Position    viewLength;
	Point       offset;
	GC          xorGC;
	GC          inverseGC[BG_SHADES];
#ifdef WINVER
	char        userName[81], scoreFile[81], bumpSound[81];
#else
	int         menu, pixmapSize;
	char       *userName, *scoreFile, *bumpSound;
	Pixel       foreground, background;
	Pixel       thingColor[MAX_START_POLYOMINOES];
	XtCallbackList select;
#endif
} TetrisPart;

typedef struct _TetrisRec {
	CorePart    core;
	TetrisPart  tetris;
} TetrisRec;

#ifdef WINVER
extern int brighter(const int light);
extern int darker(const int light);
#else
extern Pixel brighter(TetrisWidget w, Pixel pixel);
extern Pixel darker(TetrisWidget w, Pixel pixel);
#endif
extern void redoNext(TetrisWidget w);
#endif /* _TetrisP_h */
